Subject: fix for vulnerability CVE-2012-3436 for OpenTTD 0.7.0 - 0.7.5 (Denial of service (server) using ships on half tiles and landscaping.)
From: OpenTTD developer team <info@openttd.org>
Origin: backport, https://github.com/OpenTTD/OpenTTD/commit/59a3bbc https://github.com/OpenTTD/OpenTTD/commit/e07e246
Bug: https://github.com/OpenTTD/OpenTTD/issues/5254

Denial of service using ships on half tiles and landscaping.

Simple steps to reproduce the issue, and show the severity:

  start a new game. For this reproduction you do not need to start a server;
you can see the crash locally, but due to the nature of OpenTTD the crash
will also happen on the server you’re playing on with multiplayer.
  build some horizontal or vertical track at the coast, so that half of the
tile remains water or coast. The tile should either have
one corner raised with flat water on one half tile (case 1)
or two adjacent corners raised with coast on the sloped half tile (case 2)
  build a ship depot, a ship and a dock at the coast, and start the ship
  obstruct the path of the ship in a way so that it enters the tile with half
railtrack and half water
  landscape the tile by raising the water corner while the ship is on it
(only needed in case 1)
  both cases will make the ship end up on land
  remove the track using the “remove track” tool while the ship is on the tile
  server segfaults due to NULL pointer dereference.


The problem is caused by incorrectly handling the water/coast aspect of tiles
which also have railtracks on one half.
The fix adds the correct checks to the landscaping and movement code.

If you try to reproduce this with the patch applied you’ll see that,
in case 1, step 5 will deny the terraforming and in case 2 the ship simply
won’t try to enter the coast tile.

This bug was triggered incidentally by a user playing online when landscaping
near a ship. We have not seen any signs of this bug being exploited forcefully.


Index: src/rail_cmd.cpp
===================================================================
--- src/rail_cmd.cpp	(revision 24439)
+++ src/rail_cmd.cpp	(working copy)
@@ -2248,7 +2248,7 @@
 static TrackStatus GetTileTrackStatus_Track(TileIndex tile, TransportType mode, uint sub_mode, DiagDirection side)
 {
 	/* Case of half tile slope with water. */
-	if (mode == TRANSPORT_WATER && IsPlainRail(tile) && GetRailGroundType(tile) == RAIL_GROUND_WATER) {
+	if (mode == TRANSPORT_WATER && IsPlainRail(tile) && GetRailGroundType(tile) == RAIL_GROUND_WATER && IsSlopeWithOneCornerRaised(GetTileSlope(tile, NULL))) {
 		TrackBits tb = GetTrackBits(tile);
 		switch (tb) {
 			default: NOT_REACHED();
@@ -2547,6 +2547,14 @@
 	return  cost;
 }
 
+/**
+ * Test-procedure for HasVehicleOnPos to check for a ship.
+ */
+static Vehicle *EnsureNoShipProc(Vehicle *v, void *data)
+{
+	return v->type == VEH_SHIP ? v : NULL;
+}
+
 static CommandCost TerraformTile_Track(TileIndex tile, DoCommandFlag flags, uint z_new, Slope tileh_new)
 {
 	uint z_old;
@@ -2556,6 +2564,9 @@
 		/* Is there flat water on the lower halftile, that must be cleared expensively? */
 		bool was_water = (GetRailGroundType(tile) == RAIL_GROUND_WATER && IsSlopeWithOneCornerRaised(tileh_old));
 
+		/* Allow clearing the water only if there is no ship */
+		if (was_water && HasVehicleOnPos(tile, NULL, &EnsureNoShipProc)) return_cmd_error(STR_980E_SHIP_IN_THE_WAY);
+
 		_error_message = STR_1008_MUST_REMOVE_RAILROAD_TRACK;
 
 		/* First test autoslope. However if it succeeds we still have to test the rest, because non-autoslope terraforming is cheaper. */

